Crate starkom_ff

Crate starkom_ff 

Finite Field Infrastructure

CI crates.io license

Overview

This crate provides the necessary infrastructure to work with finite fields for cryptographic applications, including prime fields.

The crate is freely inspired by ff with only a few minor improvements, and it’s published as starkom-ff.

For reference and testing purposes, this crate also provides an implementation of the BLS12-381 scalar field (order 0x73eda753299d7d483339d80809a1d80553bda402fffe5bfeffffffff00000001) based on our own Field256 and PrimeField256 traits.

Modules

bls12_381
helpers

Traits

Field
A finite field.
Field64
A ~64-bit Field.
Field256
A ~256-bit Field.
PrimeField
A Field whose order is a prime number.
PrimeField64
A ~64-bit prime field.
PrimeField256
A ~256-bit prime field.
ThreeAdicField
Describes a prime field with a (3^T)-th root of unity.