Finite Field Infrastructure
Overview
This crate provides the necessary infrastructure to work with finite fields for cryptographic applications, including prime fields.
The crate is freely inspired by ff with only a few minor
improvements, and it’s published as starkom-ff.
For reference and testing purposes, this crate also provides an implementation of the BLS12-381
scalar field (order 0x73eda753299d7d483339d80809a1d80553bda402fffe5bfeffffffff00000001) based on
our own Field256 and PrimeField256 traits.
Modules
Traits
- Field
- A finite field.
- Field64
- A ~64-bit
Field. - Field256
- A ~256-bit
Field. - Prime
Field - A
Fieldwhose order is a prime number. - Prime
Field64 - A ~64-bit prime field.
- Prime
Field256 - A ~256-bit prime field.
- Three
Adic Field - Describes a prime field with a (3^T)-th root of unity.